#include "avr_gl.h"

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// To calculate sin and cosin (Tables are much faster than the library)
// 62 points because of 2 Pi = 6,2..  (doesn't make any sense to me, boy...)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
const int SIN[64]={
0,25,50,75,99,122,144,164,183,200,215,228,238,246,252,255,255,253,249,242,232,
220,206,190,172,153,131,109,85,61,36,10,-14,-40,-65,-89,-113,-135,-156,-176,
-193,-209,-223,-234,-243,-250,-254,-255,-255,-251,-245,-237,-226,-213,-197,
-180,-161,-140,-118,-95,-71,-46,-21};

const int COS[64]={ 255,254,250,244,235,224,211,195,178,159,138,116,92,68,43,18,-7,-32,
-58,-82,-106,-129,-150,-170,-188,-205,-219,-231,-241,-248,-253,-255,-255,-252,
-247,-239,-229,-217,-202,-185,-167,-147,-125,-102,-78,-53,-28,
-3,22,47,72,96,119,141,162,181,198,213,226,237,245,251,255};

void glLoadIdentity()
{
	glRotationVector[0]		= 0;
	glRotationVector[1]		= 0;
	glRotationVector[2]		= 0;

	glTranslationVector[0] 	= 0;
	glTranslationVector[1] 	= 0;
	glTranslationVector[2] 	= 0;

	glScaleVector[0] 		= 0;
	glScaleVector[1] 		= 0;
	glScaleVector[2] 		= 0;

	// Rotation-Matrix
	// calculates the new position of the single vectors
	glRotationMatrix[1][1] 	= ((long) COS[glRotationVector[1]] * COS[glRotationVector[2]]) / 256;
	glRotationMatrix[2][1] 	= ((long)-COS[glRotationVector[1]] * SIN[glRotationVector[2]]) / 256;
	glRotationMatrix[3][1] 	= SIN[glRotationVector[1]];

	glRotationMatrix[1][2] 	=  ((long)COS[glRotationVector[0]] * SIN[glRotationVector[2]]) / 256  + ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[1]] * COS[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[2][2] 	=  ((long)COS[glRotationVector[0]] * COS[glRotationVector[2]]) / 256  - ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[1]] * SIN[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[3][2] 	=  ((long) -SIN[glRotationVector[0]] * COS[glRotationVector[1]] ) / 256;

	glRotationMatrix[1][3] 	=  ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[2]]) / 256  - ((long)COS[glRotationVector[0]] * SIN[glRotationVector[1]] * COS[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[2][3] 	=  ((long)SIN[glRotationVector[0]] * COS[glRotationVector[2]]) / 256  + ((long)COS[glRotationVector[0]] * SIN[glRotationVector[1]] * SIN[glRotationVector[2]]) / ((long)256 * 256);

	glRotationMatrix[3][3] 	=  ((long)COS[glRotationVector[0]] * COS[glRotationVector[1]]) / 256;
}

void glTranslate3i(int8 x, int8 y, int8 z)
{
	glTranslationVector[0] 	= x;
	glTranslationVector[1] 	= y;
	glTranslationVector[2] 	= z;
}

void glRotate3i(int8 x, int8 y, int8 z)
{
	int i, a;
	char buffer[10];
	glRotationVector[0] 	= x%62;
	glRotationVector[1] 	= y%62;
	glRotationVector[2] 	= z%62;

	// Rotation-Matrix
	// calculates the new position of the single vectors
	glRotationMatrix[1][1] 	= ((long) COS[glRotationVector[1]] * COS[glRotationVector[2]]) / 256;
	glRotationMatrix[2][1] 	= ((long)-COS[glRotationVector[1]] * SIN[glRotationVector[2]]) / 256;
	glRotationMatrix[3][1] 	= SIN[glRotationVector[1]];

	glRotationMatrix[1][2] 	=  ((long)COS[glRotationVector[0]] * SIN[glRotationVector[2]]) / 256  + ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[1]] * COS[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[2][2] 	=  ((long)COS[glRotationVector[0]] * COS[glRotationVector[2]]) / 256  - ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[1]] * SIN[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[3][2] 	=  ((long) -SIN[glRotationVector[0]] * COS[glRotationVector[1]] ) / 256;

	glRotationMatrix[1][3] 	=  ((long)SIN[glRotationVector[0]] * SIN[glRotationVector[2]]) / 256  - ((long)COS[glRotationVector[0]] * SIN[glRotationVector[1]] * COS[glRotationVector[2]]) / ((long)256 * 256);
	glRotationMatrix[2][3] 	=  ((long)SIN[glRotationVector[0]] * COS[glRotationVector[2]]) / 256  + ((long)COS[glRotationVector[0]] * SIN[glRotationVector[1]] * SIN[glRotationVector[2]]) / ((long)256 * 256);

	glRotationMatrix[3][3] 	=  ((long)COS[glRotationVector[0]] * COS[glRotationVector[1]]) / 256;
}

void glScale3i(int8 x, int8 y, int8 z)
{
	glScaleVector[0] 		= x;
	glScaleVector[1] 		= y;
	glScaleVector[2] 		= z;
}

// Shurely not not never used... :P
// This function would require every object to be stored before drawing it.
void glFlush()
{

}

void glVertex(int8 posX, int8 posY, int8 posZ, uint8 show)
{
	int x1 = posX;
	int y1 = posY;
	int z1 = posZ;
	int xr1;
	int yr1;
	char buffer[10];

    // Convert the vector through the Rotation-Matrix
    xr1 = (glRotationMatrix[1][1] * x1) / MUL + (glRotationMatrix[2][1] * y1) / MUL + (glRotationMatrix[3][1] * z1) / MUL;
    yr1 = (glRotationMatrix[1][2] * x1) / MUL + (glRotationMatrix[2][2] * y1) / MUL + (glRotationMatrix[3][2] * z1) / MUL;

	// Draws the vertex
    glcd_pixel( xr1 + glTranslationVector[0], yr1 + glTranslationVector[1], show);
}


void glLine(int8 posX, int8 posY, int8 posZ, int8 posX2, int8 posY2, int8 posZ2, uint8 show)
{
	int x1 = posX, x2 = posX2;
	int y1 = posY, y2 = posY2;
	int z1 = posZ, z2 = posZ2;
	int xr1, xr2;
	int yr1, yr2;
	char buffer[10];

    // Convert the vector through the Rotation-Matrix
    xr1 = (glRotationMatrix[1][1] * x1) / MUL + (glRotationMatrix[2][1] * y1) / MUL + (glRotationMatrix[3][1] * z1) / MUL;
    yr1 = (glRotationMatrix[1][2] * x1) / MUL + (glRotationMatrix[2][2] * y1) / MUL + (glRotationMatrix[3][2] * z1) / MUL;
    xr2 = (glRotationMatrix[1][1] * x2) / MUL + (glRotationMatrix[2][1] * y2) / MUL + (glRotationMatrix[3][1] * z2) / MUL;
    yr2 = (glRotationMatrix[1][2] * x2) / MUL + (glRotationMatrix[2][2] * y2) / MUL + (glRotationMatrix[3][2] * z2) / MUL;

	// Draws the line
    glcd_line( xr1 + glTranslationVector[0], yr1 + glTranslationVector[1], xr2 + glTranslationVector[0], yr2 + glTranslationVector[1], show);
}


void glTriangle(vertexBuffer vertex1, vertexBuffer vertex2, vertexBuffer vertex3, uint8 show)
{

}


void glPyramid(int8 posX, int8 posY, int8 sizeX, int8 sizeY, uint8 show)
{
			glLine(20, 0,20, 0,40, 0, show);
			glLine(20, 0,20,40,40, 0, show);
			glLine(20, 0,20,40,40,40, show);
			glLine(20, 0,20, 0,40,40, show);
			glLine( 0,40, 0,40,40, 0, show);
			glLine(40,40, 0,40,40,40, show);
			glLine(40,40,40, 0,40,40, show);
			glLine( 0,40,40, 0,40, 0, show);	
}

void glQuad(int8 posX, int8 posY, int8 sizeX, int8 sizeY, uint8 show)
{

}

void glCube(int8 posX, int8 posY, int8 posZ, int8 sizeX, int8 sizeY, int8 sizeZ, uint8 show)
{

}

void glSphere(int8 posX, int8 posY, int8 posZ, int8 radius, int8 resolution, uint8 show)
{

}


// routine to draw and calc 3d cube
void gl_cube(int8 posX, int8 posY, int8 posZ, int8 sizeX, int8 sizeY, int8 sizeZ, uint8 show)
{
/*    
	int newx[8];                   	// translated screen x co-ordinates for vertex
    int newy[8];                   	// translated screen y co-ordinates for vertex
    int i, loop;                    	// temp variable for loops
    float xt, yt, zt, x, y, z, sinax, cosax, sinay, cosay, sinaz, cosaz, vertex;  // lots of work variables
    float xpos=0;				// position for object in 3d space, in x
    float ypos=0;				// y
    float zpos=0;				// and z values
    float rotx=0;                  	// starting amount of x rotation
    float roty=0;                 		// starting amount of y rotation
    float rotz=0;                		// starting amount of z rotation

    for (loop=0; loop <= 100; loop++)	// rotate the cube 100 times
    {
        xpos = xpos + 0.0;			// move the object
        ypos = ypos + 0.0;			// it would wander off screen
        zpos = zpos + 0.0;			// really quick, so leave it centered

        rotx = rotx + 0.5;                	// rotate the cube on X axis
        roty = roty + 0.5;                	// and on its y axis
        rotz = rotz + 0.0;                	// dont bother with z or it gets confusing

        sinax = sin(rotx);			// precalculate the sin and cos values
        cosax = cos(rotx);			// for the rotation as this saves a

        sinay = sin(roty);			// little time when running as we
        cosay = cos(roty);			// call sin and cos less often

        sinaz = sin(rotz);			// they are slow routines
        cosaz = cos(rotz);			// and we dont want slow!

        for (i = 0; i < 8; i++)           	// translate 3d vertex position to 2d screen position
        {
            x = aa[i];                	// get x for vertex i
            y = bb[i];                	// get y for vertex i
            z = cc[i];                	// get z for vertex i

            yt = y * cosax - z * sinax;	// rotate around the x axis
            zt = y * sinax + z * cosax;	// using the Y and Z for the rotation
            y = yt;
            z = zt;

            xt = x * cosay - z * sinay;	// rotate around the Y axis
            zt = x * sinay + z * cosay;	// using X and Z
            x = xt;
            z = zt;

            xt = X * cosaz - y * sinaz;	// finaly rotate around the Z axis
            yt = X * sinaz + y * cosaz;	// using X and Y
            x = xt;
            y = yt;

            x = x + xpos;			// add the object position offset
            y = y + ypos;			// for both x and y
            z = z + OFFSETZ - zpos;		// as well as Z

            newx[i] = (x * 64 / z) + OFFSETX;	// translate 3d to 2d coordinates for screen
            newy[i] = (y * 64 / z) + OFFSETY;	// drawing so we can see the cube
        }

        delay_ms(20);			// delay for a while to allow looking at the cube
        clearscreen();		// clear the screen to remove old cube

        for (i = 0; i < 12; i++)		// draw the lines that make up the object
        {
            vertex = ff[i] - 1;         // temp = start vertex for this line
            sx = newx[vertex];        // set line start x to vertex[i] x position
            sy = newy[vertex];        // set line start y to vertex[i] y position
            vertex = gg[i]-1;         // temp = end vertex for this line
            ex = newx[vertex];	// set line end x to vertex[i+1] x position
            ey = newy[vertex];	// set line end y to vertex[i+1] y position
            lcdline();		// draw the line between these 2 vertex
        }
    }
	*/
}


void glClear()
{
	// Clearing the Display
	glcd_clear_graph();
	glcd_clear_text();
}
